-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}
local htmlTexture = require("htmlTexture")

local hasSentMapData = false
local vehicleID = obj:getID()

local menuCol = 2

local appLeft = 1
local appRight = 2

local lastApp = 1

local appLeftOption = 1
local appRightOption = 1
local centerOption = 1 --0: normal , 1: sport , 2: race

local ecoModeEnabled = true

local escController


local cruiseOn = false
local cruiseTarget = 0;
local cruiseCurThrottle = 0;

local cruisePause = false;
local cruisePauseEnd = false;

local function onInit()

	htmlTexture.create("@R20Cluster", "local://local/vehicles/Raven_R20/R20_gauges_screen.html", 600, 150, 30, 'automatic')
	--htmlTexture.create("@R20HUD", "local://local/vehicles/Raven_R20/R20_HUD.html", 7, 3, 30, 'automatic')
	
	menuCol = 2 --Reset the menu column back to the middle
	
	--Init for Navigation system
	hasSentMapData = false
	obj:queueGameEngineLua("extensions.load('ui/uinavi')")
	obj:queueGameEngineLua(string.format('extensions.ui_uinavi.requestVehicleDashboardMap(%q)', "@R20Cluster"))
	
	hasSentMapData = true
	
	--Init function for dashboard
    htmlTexture.call("@R20Cluster", "init")
	
	--Init the throttle map and select the correct dashboard
	if(centerOption==0)then --Normal
		htmlTexture.call("@R20Cluster", "DashNormal")
		ecoModeEnabled = true
	end
	if(centerOption==1)then --Sport
		htmlTexture.call("@R20Cluster", "DashSport")
		ecoModeEnabled = false
	end
	if(centerOption==2)then --Race
		htmlTexture.call("@R20Cluster", "DashRace")
		ecoModeEnabled = false
	end
	
	htmlTexture.call("@R20Cluster", "ShowLeft", appLeft)
	htmlTexture.call("@R20Cluster", "ShowRight", appRight)
	
	escController = controller.getController('esc')
	
	--Init for cruise control
	cruiseOn = false
	cruisePause = false
	cruisePauseEnd = false
	
end

local function onReset()
    onInit()
end


local timing = false

--Dashboard Apps 

--Timer
local function appTimer(input)

	--If timing is enabled
	if(timing)then
		htmlTexture.call("@R20Cluster", "updateTimer")
	end
	
	--If up has been pressed
	if(input < 0)then
		htmlTexture.call("@R20Cluster", "resetTimer")
		timing = true
	end
	
	--If down has been pressed
	if(input > 0)then
		if(timing == true) then
			timing = false
		else
			htmlTexture.call("@R20Cluster", "resetTimer")
			htmlTexture.call("@R20Cluster", "updateTimer")
		end
	end
end


--G-Force App
local function appG()
	local gx = sensors.gx / 9.81
	local gy = sensors.gy / 9.81
	local gz = sensors.gz / 9.81
	
	htmlTexture.call("@R20Cluster", "updateGForce", {gx=gx,gy=gy,gz=gz})
end


--Navi App
local function appNavi(input)
    if hasSentMapData then
	
		local zoomMult = 1 - (input * 0.1)
		
        local pos = obj:getPosition()
		local rotation = math.deg(obj:getDirection()) + 180
		local speed = electrics.values.airspeed * 3.6
        local zoom = math.min( 300 + speed * 1.5, 500)
	
		local data = {x = pos.x, y = pos.y, rotation = rotation, zoom = zoom, zoomMult = zoomMult}
		htmlTexture.call("@R20Cluster", "map.updateData", data)
	end	
end


--Cruise Control App


local function appCruise(input) --Runs the cruise control UI
	--if up has been pressed, set cruise control to current speed
	if(input < 0)then
		cruiseOn = true
		cruiseTarget = math.floor( electrics.values.wheelspeed * 2.25 or 0) 
	end
	--if down has been pressed, disable the cruise control
	if(input > 0)then
		cruiseOn = false
	end
	
	htmlTexture.call("@R20Cluster", "updateCruise", {speed = cruiseTarget, cruiseOn = cruiseOn})
end

local function RunCruiseControl() --Runs the cruise control if it is enabled
	if (cruiseOn == true) then
		--###Interesting algorithm stuff###
		--=================================
		
		local wSpeed = math.floor( electrics.values.wheelspeed * 2.25 or 0) 
		local cruiseDiff = cruiseTarget - ( electrics.values.wheelspeed * 2.25 or 0)
		
		cruiseCurThrottle = cruiseDiff * 0.2
		
		--The above cruise control is usually pretty great, but when the going gets tuff it starts to lag behind, the code below detects slacking, calculates, and administers an appropriately sized kick in the balls to make sure it keeps up.
		if (cruiseDiff / cruiseTarget < 0.9 and cruiseDiff / cruiseTarget > 0.0) then
			cruiseCurThrottle = cruiseCurThrottle * ((cruiseDiff / cruiseTarget) + 1)
		end	
        
		--Auto Brakes the vehicle if required when going downhill
		if (cruisePause == false) then
			if(cruisePauseEnd == false) then
				if ( cruiseDiff / cruiseTarget < -0.05) then
					electrics.values.brake = - cruiseCurThrottle - 0.05
				end
			else
				if ( wSpeed < (cruiseTarget + 1) ) then
					cruisePauseEnd = false
				end
				electrics.values.brake = 0.1
			end
		end
		
		
		--###Boring Stuff###
		--==================
		
		--Keeps the applied throttle always within limits
		if ( cruiseCurThrottle > 1) then
			cruiseCurThrottle = 1
		end
		if ( cruiseCurThrottle < 0) then
			cruiseCurThrottle = 0
		end
		
		--Disables Cruise Control if brakes are applied
		if( electrics.values.brake_input > 0) then
			cruiseOn = false
			cruiseOffRoad = false
			menuOption = -1
		end
		
		--Pauses Cruise Control if player presses clutch in
		if( electrics.values.clutch_input > 0 or electrics.values.throttle_input > 0) then
			cruisePause = true;
		else
			if( cruisePause == true) then
				cruisePauseEnd = true
			end
			cruisePause = false
		end	
		
		--Adjusts throttle based on clutch to allow gear changes
		--cruiseCurThrottle = cruiseCurThrottle * ( 1 - (electrics.values.clutch * 0.8) )
		
		if (cruisePause == false) then
			electrics.values.throttle = cruiseCurThrottle
		end
	end
end


--Data Display App
local function appData()

	local pressure = {}
	for _,wd in pairs(v.data.wheels) do
		table.insert(pressure, obj:getGroupPressure(v.data.pressureGroups[wd.pressureGroup]))
	end

	local oilTemp = (electrics.values.oiltemp or 0)
	local waterTemp = (electrics.values.watertemp or 0)
	

	htmlTexture.call("@R20Cluster", "updateData", {pressure=pressure, oilTemp=oilTemp, waterTemp=waterTemp})
end







local function runEVMode(speed)
	
	--If travelling below defined speed, limit the throttle application
	local maxSpeed = 120
	
	if(speed < maxSpeed) then
		electrics.values.throttle = (electrics.values.throttle * 0.6) + (electrics.values.throttle * 0.4 * ( speed / maxSpeed))
	end
	
end

local function runEcoMode()
	electrics.values.throttle = electrics.values.throttle^2
	
	--Bonus throttle map!
	--Creates an S-Shaped mapping
	--But that wasn't very good, because in 6th gear at 70mph, you only need very little throttle to maintain speed
	--And an S shape made it a little too edgy at 1-5% output power
	
	--throttle = electrics.values.throttle
	--electrics.values.throttle = 2.5 * ((throttle - 0.3715) * 0.3) / (0.3 - math.abs(throttle - 0.3715) + 1) + 0.3

	
end
local function runNormalMode()
	electrics.values.throttle = electrics.values.throttle^1.3
end










local function updateGFX(dt)
--Dashboard controls
	--Get inputs for Dashboard
	local menuRight = electrics.values["menuRight"]
	local menuLeft = electrics.values["menuLeft"]
	local menuUp = electrics.values["menuDown"]
	local menuDown = electrics.values["menuUp"]
	
	
	--Prevent double clicks by resetting electrics values
	electrics.values["menuRight"] = false
	electrics.values["menuLeft"] = false
	electrics.values["menuDown"] = false
	electrics.values["menuUp"] = false
	
	--Apply changes if any have been made
		
	--Up and Down selection (for each column)
	if(menuUp == 1) then
		if(menuCol == 0) then --Left App Select
			appLeft = appLeft + 1
			if(appLeft > 4) then
				appLeft = 0
			end
			htmlTexture.call("@R20Cluster", "SelectApp", appLeft)
		end
		
		if(menuCol == 1) then --Left Screen
			appLeftOption = appLeftOption + 1
		end
		
		if(menuCol == 2) then --Middle Screen
			centerOption = centerOption + 1
			if(centerOption > 2) then
				centerOption = 2
			end
			
			--Change the screen
			if(centerOption==0)then --Normal
				htmlTexture.call("@R20Cluster", "DashNormal")
				ecoModeEnabled = true
				if(escController) then
					escController.setESCMode(1)
				end
			end
			if(centerOption==1)then --Sport
				htmlTexture.call("@R20Cluster", "DashSport")
				ecoModeEnabled = false
				if(escController) then
					escController.setESCMode(2)
				end
			end
			if(centerOption==2)then --Race
				htmlTexture.call("@R20Cluster", "DashRace")
				ecoModeEnabled = false
				if(escController) then
					escController.setESCMode(3)
				end
			end
		end
		
		if(menuCol == 3) then --Right Screen
			appRightOption = appRightOption + 1
		end
		
		if(menuCol == 4) then --Right App Select
			appRight = appRight + 1
			if(appRight > 4) then
				appRight = 0
			end
			htmlTexture.call("@R20Cluster", "SelectApp", appRight)
		end
	end
	
	if(menuDown == 1) then
		if(menuCol == 0) then --Left App Select
			appLeft = appLeft - 1
			if(appLeft < 0) then
				appLeft = 4
			end
			htmlTexture.call("@R20Cluster", "SelectApp", appLeft)
		end
		
		if(menuCol == 1) then --Left Screen
			appLeftOption = appLeftOption - 1
		end
		
		if(menuCol == 2) then --Middle Screen
			centerOption = centerOption - 1
			if(centerOption < 0) then
				centerOption = 0
			end
			
			--Change the screen
			if(centerOption==0)then --Normal
				htmlTexture.call("@R20Cluster", "DashNormal")
				ecoModeEnabled = true
				if(escController) then
					escController.setESCMode(1)
				end
			end
			if(centerOption==1)then --Sport
				htmlTexture.call("@R20Cluster", "DashSport")
				ecoModeEnabled = false
				if(escController) then
					escController.setESCMode(2)
				end
			end
			if(centerOption==2)then --Race
				htmlTexture.call("@R20Cluster", "DashRace")
				ecoModeEnabled = false
				if(escController) then
					escController.setESCMode(3)
				end
			end
		end
		
		if(menuCol == 3) then --Right Screen
			appRightOption = appRightOption - 1
		end
		
		if(menuCol == 4) then --Right App Select
			appRight = appRight - 1
			if(appRight < 0) then
				appRight = 4
			end
			htmlTexture.call("@R20Cluster", "SelectApp", appRight)
		end
	end
	
	--User moves selection to the right
	if(menuRight == 1) then
		menuCol = menuCol + 1
		
		
		if(menuCol == 4) then --If right app select reached
			htmlTexture.call("@R20Cluster", "HideRight", appRight)
			htmlTexture.call("@R20Cluster", "SelectApp", appRight)
			lastApp = appRight
		end
		
		
		if(menuCol == 1) then --If left app select is closed
			if(appRight == appLeft) then
				--Swap options around to maintain state
				local swap = appLeftOption
				appLeftOption = appRightOption
				appRightOption = swap
			
				appRight = lastApp
				htmlTexture.call("@R20Cluster", "ShowRight", appRight)
			else
				--Reset the input option
				appLeftOption = 0;
			end
			htmlTexture.call("@R20Cluster", "ShowLeft", appLeft)
		end
		
		
		if(menuCol > 4) then --If right app select is closed
			if(appRight == appLeft) then
				--Swap options around to maintain state
				local swap = appLeftOption
				appLeftOption = appRightOption
				appRightOption = swap
			
				appLeft = lastApp
				htmlTexture.call("@R20Cluster", "ShowLeft", appLeft)
			else
				--Reset the input option
				appRightOption = 0;
			end
			htmlTexture.call("@R20Cluster", "ShowRight", appRight)	
			menuCol = 3
		end
		htmlTexture.call("@R20Cluster", "ColSelect", menuCol)
	end
	
	
	--User moves selection to the left
	if(menuLeft == 1) then
		menuCol = menuCol - 1
		
		if(menuCol == 0) then --If left app select reached
			htmlTexture.call("@R20Cluster", "HideLeft", appLeft)
			htmlTexture.call("@R20Cluster", "SelectApp", appLeft)
			lastApp = appLeft
		end
		
		
		if(menuCol == 3) then --If right app select is closed
			if(appRight == appLeft) then
				--Swap options around to maintain state
				local swap = appLeftOption
				appLeftOption = appRightOption
				appRightOption = swap
				
				appLeft = lastApp
				htmlTexture.call("@R20Cluster", "ShowLeft", appLeft)
			else
				--Reset the input option
				appLeftOption = 0;
			end
			htmlTexture.call("@R20Cluster", "ShowRight", appRight)	
		end
		
		
		if(menuCol < 0) then --Limit end
			if(appRight == appLeft) then
				--Swap options around to maintain state
				local swap = appLeftOption
				appLeftOption = appRightOption
				appRightOption = swap
			
				appRight = lastApp
				htmlTexture.call("@R20Cluster", "ShowRight", appRight)
			else
				--Reset the input option
				appLeftOption = 0;
			end
			htmlTexture.call("@R20Cluster", "ShowLeft", appLeft)
			menuCol = 1
		end
		htmlTexture.call("@R20Cluster", "ColSelect", menuCol)
	end
	
	
--Dashboard Core Data
	local speed = math.floor( electrics.values.wheelspeed * 2.25 or 0)
	local rpm = math.floor( electrics.values.rpm + 0.5 or 0)
	local gear = (electrics.values.gear_M or 0)
	local esc = (electrics.values.esc or 0)
	local tcs = (electrics.values.tcs or 0)
	
--Run Dashboard Apps Left
	if(appLeft == 0) then 
		appTimer(appLeftOption) 
		appLeftOption = 0
	end
	if(appLeft == 1) then appG() 
	end
	if(appLeft == 2) then 
		if(appLeftOption <= -9) then
			appLeftOption = -9
		end
		if(appLeftOption >= 9) then
			appLeftOption = 9
		end
		appNavi(appLeftOption) 
	end
	if(appLeft == 3) then 
		appCruise(appLeftOption) 
		appLeftOption = 0
	end
	if(appLeft == 4) then appData() 
	end
	
--Run Dashboard Apps Right
	if(appRight == 0) then 
		appTimer(appRightOption) 
		appRightOption = 0
	end
	if(appRight == 1) then appG() 
	end
	if(appRight == 2) then 
		if(appRightOption <= -9) then
			appRightOption = -9
		end
		if(appRightOption >= 9) then
			appRightOption = 9
		end
		appNavi(appRightOption) 
	end
	if(appRight == 3) then 
		appCruise(appRightOption) 
		appRightOption = 0
	end
	if(appRight == 4) then appData() 
	end
	
	fuel = (electrics.values.fuel or 0)
	
	htmlTexture.call("@R20Cluster", "UniDataUpdate", {fuel=fuel, speed=speed, rpm=rpm, gear=gear, esc=esc, tcs=tcs, cruiseOn=cruiseOn})
	
	left = (electrics.values.signal_L or 0)
	right = (electrics.values.signal_R or 0)
	low = (electrics.values.lowbeam or 0)
	high = (electrics.values.highbeam or 0)
	fog = (electrics.values.fog or 0)
	parking = (electrics.values.parkingbrake or 0)
	
	htmlTexture.call("@R20Cluster", "LightsDataUpdate", {parking=parking, left=left, right=right, low=low, high=high, fog=fog})
	

--Detirmine if the vehicle has an electric motor rather than an engine
	if(electrics.values.oiltemp < 1) then
		runEVMode(speed)
	end

	
--Run Eco mode if required
	if(ecoModeEnabled == true) then
		runEcoMode()
	end
	if(centerOption == 1) then
		runNormalMode()
	end
	
--Run cruise control if enabled
	RunCruiseControl()
end






-- public interface
M.onInit    = onInit
M.onReset   = onReset
M.updateGFX = updateGFX

return M
